/*
 *	Title:		Escavalon - Interactive Fiction System
 *	Version:	Beta
 *	Created by:	Paolo Copello
 *	Home page:	http://web.tiscali.it/fanelia
 *	
 *	Copyright (C) 2003 Paolo Copello
 *	
 *	******************************************
 *	This file is part of Escavalon
 *	******************************************
 *	
 *	Escavalon is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *	
 */


// suppress "truncated debug info" warnings (VC++ specific)
#ifdef _MSC_VER
#pragma warning (disable: 4786)
#pragma warning (disable: 4788)
#endif // _MSC_VER

#include <stdexcept>

#include "AvClass.h"
#include "fnutils.h"


/////////////////////////////////////
// AvEntity
/////////////////////////////////////

AvEntity::AvEntity(int nID) 
: _nID(nID) 
{ 
	if (nID <= 0)
		throw invalid_argument("Every entity in Escavalon must have a positive ID");
}


/////////////////////////////////////
// AvTerm
/////////////////////////////////////

AvTerm::AvTerm(int nID, const string& sWords)
: AvEntity(nID)
{
	string strEp;
	Fanel::Str::Epurate(sWords, strEp);
	Fanel::Str::ToUpper(strEp);
	Fanel::Str::Tokenize(strEp, _words);
}

void AvTerm::AddWords(const string& sWords)
{
	list<string> newList;
	string strEp;
	Fanel::Str::Epurate(sWords, strEp);
	Fanel::Str::ToUpper(strEp);
	Fanel::Str::Tokenize(strEp, newList);
	_words.insert(_words.end(), newList.begin(), newList.end());
}


/////////////////////////////////////
// AvEntityWithState
/////////////////////////////////////

const AvState& AvEntityWithState::GetState() const
{
	return _states[_nCurrentStateID];
}

void AvEntityWithState::SetState(int nStateID)
{
	if (_states.Contains(nStateID))
		_nCurrentStateID = nStateID;
	else
		throw invalid_argument("Invalid State ID in SetState");
}

const string& AvEntityWithState::GetShortDesc() const
{
	return _states[_nCurrentStateID].GetShortDesc();
}
const string& AvEntityWithState::GetLongDesc() const
{
	return _states[_nCurrentStateID].GetLongDesc();
}

AvEntityWithState::AvEntityWithState(int nID, const string& sShortDesc, const string& sLongDesc)
: AvEntity(nID), _nCurrentStateID(AV_INITIAL)
{
	AvState initialState(AV_INITIAL, sShortDesc, sLongDesc);
	_states.Add(AV_INITIAL, initialState);
}

void AvEntityWithState::AddState(const AvState& state)
{
	_states.Add(state.GetID(), state);
}


/////////////////////////////////////
// AvLocation
/////////////////////////////////////



AvLocation::AvLocation(int nID, const string& sShortDesc, const string& sLongDesc,
					   int north, int south, int east, int west, int up, int down)
: AvEntityWithState(nID, sShortDesc, sLongDesc), _bVisited(false),
	_nNorth(north), _nSouth(south), _nEast(east), _nWest(west), _nUp(up), _nDown(down)
	
{
	// NO !!!
	// TODO: far fare i controlli di consistenza ad AvalonGame o AvDesigner!!!
	/*if (!IsValidLocation(north) || !IsValidLocation(south) || 
		!IsValidLocation(east) || !IsValidLocation(west) || 
		!IsValidLocation(up) || !IsValidLocation(down) )
	{
		throw invalid_argument("AvLocation: invalid link to other location");
	}*/
}

const string& AvLocation::GetDesc() const
{
	if(_bVisited)
		return GetShortDesc();
	return GetLongDesc();
}

void AvLocation::SetVisited()
{
	_bVisited = true;
}

void AvLocation::SetState(int nStateID)
{
	AvEntityWithState::SetState(nStateID);
	_bVisited = false;
}

ostream& operator << (ostream& out, const AvLocation& location)
{
	out << location.GetStateID() << " " << (int)location._bVisited << endl;
	return out;
}

istream& operator >> (istream& in, AvLocation& location)
{
	int nState;
	in >> nState;
	location.SetState(nState);
	in >> location._bVisited;
	Fanel::ClearInput(in);
	return in;
}


/*void ReadFromString(const char* buffer, AvLocation& location)
{
	int nState;
	istrstream strIn(buffer);
	strIn >> nState;
	location.SetState(nState);
	strIn >> location._bVisited;
}*/

/////////////////////////////////////
// AvObject
/////////////////////////////////////


// TODO chi fa i controlli? AvalonGame o AvDesigner

AvObject::AvObject(int nID, const string& sShortDesc, const string& sLongDesc,
				   int nInitialLocation, bool bFixed, OBJECT_TYPE type)
: AvEntityWithState(nID, sShortDesc, sLongDesc),
	_nLocation(nInitialLocation), _bFixed(bFixed), _type(type)
{
}

// TODO chi fa i controlli? AvalonGame

void AvObject::MoveTo(int nLocation)
{
	_nLocation = nLocation;
}


ostream& operator << (ostream& out, const AvObject& obj)
{
	out << obj.GetStateID() << " " << obj._nLocation << endl;
	return out;
}

istream& operator >> (istream& in, AvObject& obj)
{
	int nState;
	in >> nState;
	obj.SetState(nState);
	in >> obj._nLocation;
	Fanel::ClearInput(in);
	return in;
}


/////////////////////////////////////
// AvStrVariable and AvIntVariable
/////////////////////////////////////


ostream& operator << (ostream& out, const AvIntVariable& variable)
{
	out << variable.nVal << endl;
	return out;
}

istream& operator >> (istream& in, AvIntVariable& variable)
{
	in >> variable.nVal;
	Fanel::ClearInput(in);
	return in;
}

ostream& operator << (ostream& out, const AvStrVariable& variable)
{
	out << variable.sVal << endl;
	return out;
}

istream& operator >> (istream& in, AvStrVariable& variable)
{
	char buffer[512];
	in.getline(buffer, 512);
	variable.sVal = buffer;
	return in;
}


/////////////////////////////////////
// AvTimer
/////////////////////////////////////

AvTimer::AvTimer(int nID)
: AvEntity(nID), _nTime(-1)
{
}

void AvTimer::SetTime(int nTime)
{
	_nTime = nTime;
}

int AvTimer::Tick()
{
	_nTime--;
	if(_nTime < 0)
		_nTime = -1;
	return _nTime;
}

ostream& operator << (ostream& out, const AvTimer& variable)
{
	out << variable.GetTime() << endl;
	return out;
}

istream& operator >> (istream& in, AvTimer& variable)
{
	int nT;
	in >> nT;
	Fanel::ClearInput(in);
	variable.SetTime(nT);
	return in;
}


