/*
 *	Title:		Escavalon - Interactive Fiction System
 *	Version:	Beta
 *	Created by:	Paolo Copello
 *	Home page:	http://web.tiscali.it/fanelia
 *	
 *	Copyright (C) 2003 Paolo Copello
 *	
 *	******************************************
 *	This file is part of Escavalon
 *	******************************************
 *	
 *	Escavalon is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *	
 */


#if !defined(FANEL_Maps_427FDBA3_6CD5_4208_B99A_1570DFDA25B8)
#define FANEL_Maps_427FDBA3_6CD5_4208_B99A_1570DFDA25B8


#include <stdexcept>
#include <list>
#include <map>
#include <set>
#include <fstream>

namespace Fanel
{

template <class T>
class _IntegerMapIterator
{
public:
	typedef _IntegerMapIterator<T> _Self;
	typedef std::map<int, T> ItemMap;

	_IntegerMapIterator() { }
	_IntegerMapIterator(const ItemMap::iterator& iter) : _iter(iter) { }

	bool operator==(const _Self& __x) const { return _iter == __x._iter; }
	bool operator!=(const _Self& __x) const { return _iter != __x._iter; }
	T& operator*() const { return _iter->second; }
	T* operator->() const { return &(operator*()); }

	_Self& operator++() { 
		++_iter;
		return *this;
	}
	_Self operator++(int) { 
		_Self __tmp = *this;
		++*this;
		return __tmp;
	}

private:
	ItemMap::iterator _iter;
};


// forward declarations
template <class T> class IntegerMap;

template <class T>
std::ostream& operator << (std::ostream& out, const IntegerMap<T>& map);

template <class T>
std::istream& operator >> (std::istream& in, IntegerMap<T>& map);


/* VC++ specific */
#ifdef _MSC_VER
#define TEMPLATIZER
#else
#define TEMPLATIZER <T>
#endif // _MSC_VER

template<class T>
class IntegerMap
{
public:
	IntegerMap() { }
	~IntegerMap()  { Clear(); }

	T& operator[] (int nID);
	const T& operator[] (int nID) const;

	bool Contains(int nID) const;
	void Add(int nID, const T& item);
	void Clear();
	void GetAsList(std::list<T>& lst) const;

	typedef _IntegerMapIterator<T> iterator;
	iterator begin();
	iterator end();
private:
	typedef std::map<int, T> ItemMap;
	ItemMap _map;
	friend std::ostream& operator << TEMPLATIZER(std::ostream& out, const IntegerMap<T>& map);
	friend std::istream& operator >> TEMPLATIZER(std::istream& in, IntegerMap<T>& map);

public:
};

template <class T>
T& IntegerMap<T>::operator[] (int nID)
{
	ItemMap::iterator it = _map.find(nID);
	if (it == _map.end())
	{
		throw std::invalid_argument ("Item does not exist in IntegerMap");
	}
	return (it->second);
}

template <class T>
const T& IntegerMap<T>::operator[] (int nID) const
{
	ItemMap::const_iterator it = _map.find(nID);
	if (it == _map.end())
	{
		throw std::invalid_argument ("Item does not exist in IntegerMap");
	}
	return (it->second);
}

template <class T>
bool IntegerMap<T>::Contains(int nID) const
{
	ItemMap::const_iterator it = _map.find(nID);
	if (it == _map.end())
		return false;
	return true;
}

template <class T>
void IntegerMap<T>::Add(int nID, const T& item)
{
	if (Contains(nID))
	{
		throw std::invalid_argument ("Item already present in IntegerMap");
	}
	;
	_map.insert(std::pair <int, T> (nID, item));
}

template <class T>
void IntegerMap<T>::Clear()
{
	_map.clear();
}

template <class T>
void IntegerMap<T>::GetAsList(std::list<T>& lst) const
{
	lst.clear();
	ItemMap::const_iterator it = _map.begin();
	for(; it!=_map.end(); ++it)
	{
		lst.push_back(it->second);
	}
}

template <class T>
IntegerMap<T>::iterator IntegerMap<T>::begin()
{
	return iterator(_map.begin());
}

template <class T>
IntegerMap<T>::iterator IntegerMap<T>::end()
{
	return iterator(_map.end());
}


template <class T>
std::ostream& operator << (std::ostream& out, const IntegerMap<T>& map)
{
	out << map._map.size() << endl;
	IntegerMap<T>::ItemMap::const_iterator it = map._map.begin();
	for(; it!= map._map.end(); ++it)
	{
		out << it->second;
	}
	return out;
}

template <class T>
std::istream& operator >> (std::istream& in, IntegerMap<T>& map)
{
	int nSize;
	in >> nSize;
	Fanel::ClearInput(in);
	if(nSize != map._map.size())
		throw -1; // TODO 

	IntegerMap<T>::ItemMap::iterator it = map._map.begin();
	for(; it!= map._map.end(); ++it)
	{
		in >> it->second;
	}
	return in;
}


//////////////////////////


template<class T>
class StringMap
{
public:
	StringMap() { }
	~StringMap()  { Clear(); }

	T& operator[] (const std::string& sID);
	const T& operator[] (const std::string& sID) const;

	bool Contains(const std::string& sID) const;
	void Add(const std::string& sID, const T& item);
	void Clear();
	void GetAsList(std::list<T>& lst) const;
private:
	typedef std::map<std::string, T> ItemMap;
	ItemMap _map;
};

template <class T>
T& StringMap<T>::operator[] (const std::string& sID)
{
	ItemMap::iterator it = _map.find(sID);
	if (it == _map.end())
	{
		throw std::invalid_argument ("Item does not exist in StringMap");
	}
	return (it->second);
}

template <class T>
const T& StringMap<T>::operator[] (const std::string& sID) const
{
	ItemMap::const_iterator it = _map.find(sID);
	if (it == _map.end())
	{
		throw std::invalid_argument ("Item does not exist in StringMap");
	}
	return (it->second);
}

template <class T>
bool StringMap<T>::Contains(const std::string& sID) const
{
	ItemMap::const_iterator it = _map.find(sID);
	if (it == _map.end())
		return false;
	return true;
}

template <class T>
void StringMap<T>::Add(const std::string& sID, const T& item)
{
	if (Contains(sID))
	{
		throw std::invalid_argument ("Item already present in StringMap");
	}
	_map.insert(std::pair<std::string, T > (sID, item));
}

template <class T>
void StringMap<T>::Clear()
{
	_map.clear();
}

template <class T>
void StringMap<T>::GetAsList(std::list<T>& lst) const
{
	lst.clear();
	ItemMap::const_iterator it = _map.begin();
	for(; it!=_map.end(); ++it)
	{
		lst.push_back(it->second);
	}
}


////////////////////////////////////////////////////////


class StringSet
{
public:
	StringSet() { }
	~StringSet()  { Clear(); }

	inline bool Contains(const std::string& str) const;
	inline void Add(const std::string& str);
	inline void Add(const std::list<std::string>& strList);
	inline void Clear();
	inline void GetAsList(std::list<std::string>& lst) const;
private:
	typedef std::set<std::string> ItemSet;
	ItemSet _set;
};


bool StringSet::Contains(const std::string& str) const
{
	ItemSet::const_iterator it = _set.find(str);
	if (it == _set.end())
		return false;
	return true;
}

void StringSet::Add(const std::string& str)
{
	if (Contains(str))
	{
		throw std::invalid_argument (std::string("Item already present in StringSet: ") + str );
	}
	_set.insert(str);
}

void StringSet::Add(const std::list<std::string>& strList)
{
	std::list<std::string>::const_iterator it = strList.begin();
	for(; it!= strList.end(); ++it)
	{
		Add(*it);
	}
}

void StringSet::Clear()
{
	_set.clear();
}


void StringSet::GetAsList(std::list<std::string>& lst) const
{
	lst.clear();
	ItemSet::const_iterator it = _set.begin();
	for(; it!=_set.end(); ++it)
	{
		lst.push_back(*it);
	}
}





} // end namespace Fanel

#endif

